@@ -67,7 +67,7 @@ def eliminate_distinct_on(expression: exp.Expression) -> exp.Expression:
         order = expression.args.get("order")
 
         if order:
-            window.set("order", order.pop().copy())
+            window.set("order", order.pop())
         else:
             window.set("order", exp.Order(expressions=[c.copy() for c in distinct_cols]))
 
@@ -75,9 +75,9 @@ def eliminate_distinct_on(expression: exp.Expression) -> exp.Expression:
         expression.select(window, copy=False)
 
         return (
-            exp.select(*outer_selects)
-            .from_(expression.subquery("_t"))
-            .where(exp.column(row_number).eq(1))
+            exp.select(*outer_selects, copy=False)
+            .from_(expression.subquery("_t", copy=False), copy=False)
+            .where(exp.column(row_number).eq(1), copy=False)
         )
 
     return expression
@@ -120,7 +120,9 @@ def eliminate_qualify(expression: exp.Expression) -> exp.Expression:
             elif expr.name not in expression.named_selects:
                 expression.select(expr.copy(), copy=False)
 
-        return outer_selects.from_(expression.subquery(alias="_t")).where(qualify_filters)
+        return outer_selects.from_(expression.subquery(alias="_t", copy=False), copy=False).where(
+            qualify_filters, copy=False
+        )
 
     return expression
 
@@ -139,7 +141,7 @@ def remove_precision_parameterized_types(expression: exp.Expression) -> exp.Expr
 
 
 def unnest_to_explode(expression: exp.Expression) -> exp.Expression:
-    """Convert cross join unnest into lateral view explode (used in presto -> hive)."""
+    """Convert cross join unnest into lateral view explode."""
     if isinstance(expression, exp.Select):
         for join in expression.args.get("joins") or []:
             unnest = join.this
@@ -164,7 +166,7 @@ def unnest_to_explode(expression: exp.Expression) -> exp.Expression:
 
 
 def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp.Expression]:
-    """Convert explode/posexplode into unnest (used in hive -> presto)."""
+    """Convert explode/posexplode into unnest."""
 
     def _explode_to_unnest(expression: exp.Expression) -> exp.Expression:
         if isinstance(expression, exp.Select):
@@ -189,7 +191,7 @@ def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp
             )
 
             # we use list here because expression.selects is mutated inside the loop
-            for select in expression.selects.copy():
+            for select in list(expression.selects):
                 explode = select.find(exp.Explode)
 
                 if explode:
@@ -197,11 +199,11 @@ def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp
                     explode_alias = ""
 
                     if isinstance(select, exp.Alias):
-                        explode_alias = select.alias
+                        explode_alias = select.args["alias"]
                         alias = select
                     elif isinstance(select, exp.Aliases):
-                        pos_alias = select.aliases[0].name
-                        explode_alias = select.aliases[1].name
+                        pos_alias = select.aliases[0]
+                        explode_alias = select.aliases[1]
                         alias = select.replace(exp.alias_(select.this, "", copy=False))
                     else:
                         alias = select.replace(exp.alias_(select, ""))
@@ -228,9 +230,12 @@ def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp
 
                     alias.set("alias", exp.to_identifier(explode_alias))
 
+                    series_table_alias = series.args["alias"].this
                     column = exp.If(
-                        this=exp.column(series_alias).eq(exp.column(pos_alias)),
-                        true=exp.column(explode_alias),
+                        this=exp.column(series_alias, table=series_table_alias).eq(
+                            exp.column(pos_alias, table=unnest_source_alias)
+                        ),
+                        true=exp.column(explode_alias, table=unnest_source_alias),
                     )
 
                     explode.replace(column)
@@ -240,8 +245,10 @@ def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp
                         expressions.insert(
                             expressions.index(alias) + 1,
                             exp.If(
-                                this=exp.column(series_alias).eq(exp.column(pos_alias)),
-                                true=exp.column(pos_alias),
+                                this=exp.column(series_alias, table=series_table_alias).eq(
+                                    exp.column(pos_alias, table=unnest_source_alias)
+                                ),
+                                true=exp.column(pos_alias, table=unnest_source_alias),
                             ).as_(pos_alias),
                         )
                         expression.set("expressions", expressions)
@@ -274,10 +281,12 @@ def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp
                         size = size - 1
 
                     expression.where(
-                        exp.column(series_alias)
-                        .eq(exp.column(pos_alias))
+                        exp.column(series_alias, table=series_table_alias)
+                        .eq(exp.column(pos_alias, table=unnest_source_alias))
                         .or_(
-                            (exp.column(series_alias) > size).and_(exp.column(pos_alias).eq(size))
+                            (exp.column(series_alias, table=series_table_alias) > size).and_(
+                                exp.column(pos_alias, table=unnest_source_alias).eq(size)
+                            )
                         ),
                         copy=False,
                     )
@@ -374,6 +383,89 @@ def eliminate_semi_and_anti_joins(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
+def eliminate_full_outer_join(expression: exp.Expression) -> exp.Expression:
+    """
+    Converts a query with a FULL OUTER join to a union of identical queries that
+    use LEFT/RIGHT OUTER joins instead. This transformation currently only works
+    for queries that have a single FULL OUTER join.
+    """
+    if isinstance(expression, exp.Select):
+        full_outer_joins = [
+            (index, join)
+            for index, join in enumerate(expression.args.get("joins") or [])
+            if join.side == "FULL"
+        ]
+
+        if len(full_outer_joins) == 1:
+            expression_copy = expression.copy()
+            expression.set("limit", None)
+            index, full_outer_join = full_outer_joins[0]
+            full_outer_join.set("side", "left")
+            expression_copy.args["joins"][index].set("side", "right")
+            expression_copy.args.pop("with", None)  # remove CTEs from RIGHT side
+
+            return exp.union(expression, expression_copy, copy=False)
+
+    return expression
+
+
+def move_ctes_to_top_level(expression: exp.Expression) -> exp.Expression:
+    """
+    Some dialects (e.g. Hive, T-SQL, Spark prior to version 3) only allow CTEs to be
+    defined at the top-level, so for example queries like:
+
+        SELECT * FROM (WITH t(c) AS (SELECT 1) SELECT * FROM t) AS subq
+
+    are invalid in those dialects. This transformation can be used to ensure all CTEs are
+    moved to the top level so that the final SQL code is valid from a syntax standpoint.
+
+    TODO: handle name clashes whilst moving CTEs (it can get quite tricky & costly).
+    """
+    top_level_with = expression.args.get("with")
+    for node in expression.find_all(exp.With):
+        if node.parent is expression:
+            continue
+
+        inner_with = node.pop()
+        if not top_level_with:
+            top_level_with = inner_with
+            expression.set("with", top_level_with)
+        else:
+            if inner_with.recursive:
+                top_level_with.set("recursive", True)
+
+            top_level_with.expressions.extend(inner_with.expressions)
+
+    return expression
+
+
+def ensure_bools(expression: exp.Expression) -> exp.Expression:
+    """Converts numeric values used in conditions into explicit boolean expressions."""
+    from sqlglot.optimizer.canonicalize import ensure_bools
+
+    def _ensure_bool(node: exp.Expression) -> None:
+        if (
+            node.is_number
+            or node.is_type(exp.DataType.Type.UNKNOWN, *exp.DataType.NUMERIC_TYPES)
+            or (isinstance(node, exp.Column) and not node.type)
+        ):
+            node.replace(node.neq(0))
+
+    for node, *_ in expression.walk():
+        ensure_bools(node, _ensure_bool)
+
+    return expression
+
+
+def unqualify_columns(expression: exp.Expression) -> exp.Expression:
+    for column in expression.find_all(exp.Column):
+        # We only wanna pop off the table, db, catalog args
+        for part in column.parts[:-1]:
+            part.pop()
+
+    return expression
+
+
 def preprocess(
     transforms: t.List[t.Callable[[exp.Expression], exp.Expression]],
 ) -> t.Callable[[Generator, exp.Expression], str]:
@@ -392,7 +484,7 @@ def preprocess(
     def _to_sql(self, expression: exp.Expression) -> str:
         expression_type = type(expression)
 
-        expression = transforms[0](expression.copy())
+        expression = transforms[0](expression)
         for t in transforms[1:]:
             expression = t(expression)
 
